/*
 * @copyright (c) 2008, Hedspi, Hanoi University of Technology
 * @author Huu-Duc Nguyen
 * @version 1.0
 */

#include <stdlib.h>
#include <ctype.h>
#include "token.h"

struct {
	char string[MAX_IDENT_LEN + 1];
	TokenType tokenType;
} keywords[KEYWORDS_COUNT] = {
	{"PROGRAM", KW_PROGRAM},
	{"CONST", KW_CONST},
	{"TYPE", KW_TYPE},
	{"VAR", KW_VAR},
	{"INTEGER", KW_INTEGER},
	{"CHAR", KW_CHAR},
	{"ARRAY", KW_ARRAY},
	{"OF", KW_OF},
	{"FUNCTION", KW_FUNCTION},
	{"PROCEDURE", KW_PROCEDURE},
	{"BEGIN", KW_BEGIN},
	{"END", KW_END},
	{"CALL", KW_CALL},
	{"IF", KW_IF},
	{"THEN", KW_THEN},
	{"ELSE", KW_ELSE},
	{"WHILE", KW_WHILE},
	{"DO", KW_DO},
	{"FOR", KW_FOR},
	{"TO", KW_TO},
	{"UNTIL",KW_UNTIL},
	{"REPEAT",KW_REPEAT},
	{"DOUBLE",KW_DOUBLE},
	{"STRING",KW_STRING}
};

int keywordEq(char *kw, char *string) {
	while ((*kw != '\0') && (*string != '\0')) {
		if (*kw != *string) break;
		kw ++; string ++;
	}
	return ((*kw == '\0') && (*string == '\0'));
}

TokenType checkKeyword(char *string) {
	int i;
	for (i = 0; i < KEYWORDS_COUNT; i++){
		if (keywordEq(keywords[i].string, string)) {
			return keywords[i].tokenType;
		}
	}
	return TK_NONE;
}

Token* makeToken(TokenType tokenType, int lineNo, int colNo) {
	Token *token = (Token*)malloc(sizeof(Token));
	token->tokenType = tokenType;
	token->lineNo = lineNo;
	token->colNo = colNo;
	return token;
}

char *tokenToString(TokenType tokenType) {
	switch (tokenType) {
	case TK_NONE: return "None";
	case TK_IDENT: return "an identification";
	case TK_NUMBER: return "a number";
	case TK_CHAR: return "a constant char";
	case TK_EOF: return "end of file";
	case TK_STRING: return "a  string";

	case KW_PROGRAM: return "keyword PROGRAM";
	case KW_CONST: return "keyword CONST";
	case KW_TYPE: return "keyword TYPE";
	case KW_VAR: return "keyword VAR";
	case KW_INTEGER: return "keyword INTEGER";
	case KW_CHAR: return "keyword CHAR";
	case KW_ARRAY: return "keyword ARRAY";
	case KW_OF: return "keyword OF";
	case KW_FUNCTION: return "keyword FUNCTION";
	case KW_PROCEDURE: return "keyword PROCEDURE";
	case KW_BEGIN: return "keyword BEGIN";
	case KW_END: return "keyword END";
	case KW_CALL: return "keyword CALL";
	case KW_IF: return "keyword IF";
	case KW_THEN: return "keyword THEN";
	case KW_ELSE: return "keyword ELSE";
	case KW_WHILE: return "keyword WHILE";
	case KW_DO: return "keyword DO";
	case KW_FOR: return "keyword FOR";
	case KW_TO: return "keyword TO";
	case KW_STRING: return "keyword STRING";
	case KW_UNTIL: return "keyword UNTIL";
	case KW_REPEAT:return "keyword REPEAT";
	case KW_DOUBLE: return "keyword DOUBLE";

	case SB_SEMICOLON: return "\';\'";
	case SB_COLON: return "\':\'";
	case SB_PERIOD: return "\'.\'";
	case SB_COMMA: return "\',\'";
	case SB_ASSIGN: return "\':=\'";
	case SB_EQ: return "\'=\'";
	case SB_NEQ: return "\'!=\'";
	case SB_LT: return "\'<\'";
	case SB_LE: return "\'<=\'";
	case SB_GT: return "\'>\'";
	case SB_GE: return "\'>=\'";
	case SB_PLUS: return "\'+\'";
	case SB_MINUS: return "\'-\'";
	case SB_TIMES: return "\'*\'";
	case SB_SLASH: return "\'/\'";
	case SB_LPAR: return "\'(\'";
	case SB_RPAR: return "\')\'";
	case SB_LSEL: return "\'(.\'";
	case SB_RSEL: return "\'.)\'";
	default: return "";
	}
}
